"""Compute diurnal temperature variation."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import sys
import logging
import pathlib
from logging.config import dictConfig
from joblib import Parallel, delayed

# externals
import xarray as xr

# locals
from pysegcnn.core.utils import search_files
from pysegcnn.core.trainer import LogConfig
from pysegcnn.core.logging import log_conf
from climax.core.cli import dtr_parser

# module level logger
LOGGER = logging.getLogger(__name__)


def compute_diurnal_variation(tasmax, tasmin, target):
    # compute diurnal variation
    tmax = xr.open_dataset(tasmax, chunks={'time': int(5 * 365)})
    tmin = xr.open_dataset(tasmin, chunks={'time': int(5 * 365)})
    dtr = (tmax.tasmax - tmin.tasmin).to_dataset(name='dtr')

    # save as NetCDF
    dtr.to_netcdf(target, engine='h5netcdf')

    # clear memory
    del tmax, tmin, dtr


if __name__ == '__main__':

    # configure logging
    dictConfig(log_conf())

    # define command line argument parser
    parser = dtr_parser()

    # parse command line arguments
    args = sys.argv[1:]
    if not args:
        parser.print_help()
        sys.exit()
    else:
        args = parser.parse_args(args)

    # check whether the source directory exists
    if args.source.exists():
        # search files matching tasmax and tasmin
        tasmax = sorted(search_files(args.source, '^tasmax(.*).nc$'))
        tasmin = sorted(search_files(args.source, '^tasmin(.*).nc$'))

        # log list of input files
        LogConfig.init_log('Input tasmax files.')
        LOGGER.info(('\n ' + (len(__name__) + 1) * ' ').join(
                ['{}'.format(file) for file in tasmax]))

        # find same EuroCordex simulations
        LogConfig.init_log('Searching corresponding tasmin files.')
        targets = []
        matches = []
        for tmax in tasmax:
            # find the corresponding tasmin file
            tmin = pathlib.Path(str(tmax).replace('tasmax', 'tasmin'))
            if tmin not in tasmin:
                LOGGER.info('Cannot find matching files for simulation: {}'
                            .format(tmax))

                # remove simulation from list of simulations to process
                tasmax.remove(tmax)
                continue

            # target file name: diurnal variation
            target = pathlib.Path(str(tmax).replace('tasmax', 'dtr'))

            # check if output path exists
            if not target.parent.exists():
                LOGGER.info('mkdir {}'.format(target.parent))
                target.parent.mkdir(parents=True, exist_ok=True)

            # check for existing files
            if target.exists() and not args.overwrite:
                LOGGER.info('{} already exists.'.format(target))
                tasmax.remove(tmax)
            else:
                # append to files to process
                targets.append(target)

            # found corresponding tasmin file
            LOGGER.info(tmin)
            matches.append(tmin)

        # log list of files to process
        LogConfig.init_log('Processing files.')
        LOGGER.info(('\n ' + (len(__name__) + 1) * ' ').join(
                ['{}'.format(file) for file in targets]))

        # check for dry run
        if args.dry_run:
            LogConfig.init_log('Dry run: no outputs produced.')
            sys.exit()

        # compute diurnal variation in parallel
        LogConfig.init_log('Computing diurnal variation.')
        Parallel(n_jobs=2, verbose=51)(delayed(
            compute_diurnal_variation)(tmax, tmin, file) for tmax, tmin, file
            in zip(tasmax, matches, targets))

    else:
        LOGGER.info('{} does not exist.'.format(str(args.source)))
        sys.exit()
